#include "stdio.h"
#include "iostream"
#include "fstream"
#include "string"

using namespace std;


/*******************************
 * Global "Lazy style" Variables
 *******************************/
static int G1[10];
static int G2[10];
static int out[1023];
static int phase1 = -1;
static int phase2 = -1;

/****************************
 *@brief: Feedback Selections 
 ****************************/
const int Phases_1 [] = {2,3,4,5,1, 2,1,2,3,2,   3,5,6,7,8, 9,1,2,3,4,  5,6,1,4,5, 6,7,8,1,2,  3,4,5,4,1,   2,4};
const int Phases_2 [] = {6,7,8,9,9, 10,8,9,10,3, 4,6,7,8,9, 10,4,5,6,7, 8,9,3,6,7, 8,9,10,6,7, 8,9,10,10,7, 8,10};


/**************************************
 *@brief: Updates the G1 Shift Register
 **************************************/
int UpdateG1(){

  //Compute New Value
  int newVal = (G1[2]+G1[9]) % 2;

  //Compute Output
  int ret = G1[9];

  // Update Shift Register
  for(int i  = 9 ;i > 0;i--){
    G1[i] = G1[i-1];
  }
  G1[0] = newVal;

  return ret;
}

/**************************************
 *@brief: Updates the G2 Shift Register
 **************************************/
int UpdateG2(){

  // Compute New Value
  int newVal = ( G2[1] + G2[2] + G2[5] + G2[7] + G2[8] + G2[9]) % 2;
  
  // Compute Output based on PRN Selected
  int retVal = (G2[phase1 - 1] + G2[phase2 - 1]) % 2;


  // Update Shift Register
  for(int i  = 9 ;i > 0;i--){
    G2[i] = G2[i-1];
  }
  G2[0] = newVal;

  return retVal;
}

/*****************************
 *@brief: Initialize Generator
 *****************************/ 
void InitGen(){

  for(int i = 0;i < 10;i++){
    G1[i] = 1;
    G2[i] = 1;
  }
}

/***********************************
 @brief: Selects correct Phase Taps
 **********************************/
void Select(int PRN){
    phase1 = Phases_1[PRN];
    phase2 = Phases_2[PRN];
}

/**********************
 *@brief: Saves to File
 **********************/
void save_to_file(std::string name){
  char delim = ';';
  FILE * PRNFile;
  fopen_s(&PRNFile, name.c_str(), "w+");
  for (int i = 0; i < 1023; i++){
    fprintf_s(PRNFile, "%d %c", out[i], delim);
  }
  fclose(PRNFile);
  
}

int main(int argc,char**argv){
  std::cout << "Please Enter PRN Number from 1 to 37: "<<endl;
  std::cout << "Enter 0 to process all PRN codes and save them ->" << endl;
  
  int PRN; 
  std::cin >> PRN;

  if((PRN >= 1) && (PRN <= 37) ){

    Select(PRN);

    //Calculate Sequence
    InitGen();
    for(int i= 0;i < 1023;i++){
      int g1 = UpdateG1();
      int g2 = UpdateG2();
      out[i] = (g1+g2) % 2;
    }

    //Save to File ...
    std::string name = "PRN" + std::to_string(PRN) + ".txt";
    save_to_file(name);
    
  }else if (PRN == 0){

    //LOOP ACROSS ALL PRN CODES
    for (int i = 1; i < 38; i++){
      std::cout << "Processing PRN: " << i << endl;

      //Calculate Sequence
      Select(i);
      InitGen();
      for (int j = 0; j<1023; j++){
        int g1 = UpdateG1();
        int g2 = UpdateG2();
        out[j] = (g1 + g2) % 2;
        
      }

      //Save to File ...
      std::string name = "PRN" + std::to_string(i) + ".txt";
      save_to_file(name);

    }

  }

  std::cout<< "PRN: " + to_string(PRN) << " Saved to .txt" << std::endl;
  system("pause");
  return 0;
}